import { customRef, nextTick, watch } from 'vue-demi';
import { useRoute, useRouter } from 'vue-router';
import { tryOnScopeDispose, toValue } from '@vueuse/shared';

let _hash;
function useRouteHash(defaultValue, {
  mode = "replace",
  route = useRoute(),
  router = useRouter()
} = {}) {
  _hash = route.hash;
  tryOnScopeDispose(() => {
    _hash = void 0;
  });
  let _trigger;
  const proxy = customRef((track, trigger) => {
    _trigger = trigger;
    return {
      get() {
        track();
        return _hash || toValue(defaultValue);
      },
      set(v) {
        if (v === _hash)
          return;
        _hash = v === null ? void 0 : v;
        trigger();
        nextTick(() => {
          const { params, query } = route;
          router[toValue(mode)]({ params, query, hash: _hash });
        });
      }
    };
  });
  watch(
    () => route.hash,
    () => {
      _hash = route.hash;
      _trigger();
    },
    { flush: "sync" }
  );
  return proxy;
}

var __defProp$1 = Object.defineProperty;
var __getOwnPropSymbols$1 = Object.getOwnPropertySymbols;
var __hasOwnProp$1 = Object.prototype.hasOwnProperty;
var __propIsEnum$1 = Object.prototype.propertyIsEnumerable;
var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues$1 = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp$1.call(b, prop))
      __defNormalProp$1(a, prop, b[prop]);
  if (__getOwnPropSymbols$1)
    for (var prop of __getOwnPropSymbols$1(b)) {
      if (__propIsEnum$1.call(b, prop))
        __defNormalProp$1(a, prop, b[prop]);
    }
  return a;
};
const _cache$1 = /* @__PURE__ */ new WeakMap();
function useRouteParams(name, defaultValue, options = {}) {
  const {
    mode = "replace",
    route = useRoute(),
    router = useRouter(),
    transform = (value) => value
  } = options;
  if (!_cache$1.has(route))
    _cache$1.set(route, /* @__PURE__ */ new Map());
  const _params = _cache$1.get(route);
  tryOnScopeDispose(() => {
    _params.delete(name);
  });
  _params.set(name, route.params[name]);
  let _trigger;
  const proxy = customRef((track, trigger) => {
    _trigger = trigger;
    return {
      get() {
        track();
        const data = _params.get(name);
        return transform(data !== void 0 ? data : toValue(defaultValue));
      },
      set(v) {
        if (_params.get(name) === v)
          return;
        _params.set(name, v);
        trigger();
        nextTick(() => {
          const { params, query, hash } = route;
          router[toValue(mode)]({
            params: __spreadValues$1(__spreadValues$1({}, params), Object.fromEntries(_params.entries())),
            query,
            hash
          });
        });
      }
    };
  });
  watch(
    () => route.params[name],
    (v) => {
      _params.set(name, v);
      _trigger();
    },
    { flush: "sync" }
  );
  return proxy;
}

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
const _cache = /* @__PURE__ */ new WeakMap();
function useRouteQuery(name, defaultValue, options = {}) {
  const {
    mode = "replace",
    route = useRoute(),
    router = useRouter(),
    transform = (value) => value
  } = options;
  if (!_cache.has(route))
    _cache.set(route, /* @__PURE__ */ new Map());
  const _query = _cache.get(route);
  tryOnScopeDispose(() => {
    _query.delete(name);
  });
  _query.set(name, route.query[name]);
  let _trigger;
  const proxy = customRef((track, trigger) => {
    _trigger = trigger;
    return {
      get() {
        track();
        const data = _query.get(name);
        return transform(data !== void 0 ? data : toValue(defaultValue));
      },
      set(v) {
        if (_query.get(name) === v)
          return;
        _query.set(name, v);
        trigger();
        nextTick(() => {
          const { params, query, hash } = route;
          router[toValue(mode)]({
            params,
            query: __spreadValues(__spreadValues({}, query), Object.fromEntries(_query.entries())),
            hash
          });
        });
      }
    };
  });
  watch(
    () => route.query[name],
    (v) => {
      _query.set(name, v);
      _trigger();
    },
    { flush: "sync" }
  );
  return proxy;
}

export { useRouteHash, useRouteParams, useRouteQuery };
